@@ -36,7 +36,6 @@ import org.jruby.Ruby;
 import org.jruby.RubyClass;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
-import org.jruby.runtime.Arity;
 import org.jruby.runtime.Helpers;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -48,6 +47,8 @@ import org.w3c.dom.NodeList;
 
 import nokogiri.internals.HtmlDomParserContext;
 
+import static nokogiri.internals.NokogiriHelpers.getNokogiriClass;
+
 /**
  * Class for Nokogiri::HTML::Document.
  *
@@ -65,21 +66,25 @@ public class HtmlDocument extends XmlDocument {
     public HtmlDocument(Ruby ruby, RubyClass klazz) {
         super(ruby, klazz);
     }
-    
+
+    public HtmlDocument(Ruby runtime, Document document) {
+        this(runtime, getNokogiriClass(runtime, "Nokogiri::XML::Document"), document);
+    }
+
     public HtmlDocument(Ruby ruby, RubyClass klazz, Document doc) {
         super(ruby, klazz, doc);
     }
 
     @JRubyMethod(name="new", meta = true, rest = true, required=0)
-    public static IRubyObject rbNew(ThreadContext context, IRubyObject klazz,
-                                    IRubyObject[] args) {
+    public static IRubyObject rbNew(ThreadContext context, IRubyObject klazz, IRubyObject[] args) {
+        final Ruby runtime = context.runtime;
         HtmlDocument htmlDocument;
         try {
-            Document docNode = createNewDocument();
-            htmlDocument = (HtmlDocument) NokogiriService.HTML_DOCUMENT_ALLOCATOR.allocate(context.getRuntime(), (RubyClass) klazz);
-            htmlDocument.setDocumentNode(context, docNode);
+            Document docNode = createNewDocument(runtime);
+            htmlDocument = (HtmlDocument) NokogiriService.HTML_DOCUMENT_ALLOCATOR.allocate(runtime, (RubyClass) klazz);
+            htmlDocument.setDocumentNode(context.runtime, docNode);
         } catch (Exception ex) {
-            throw context.getRuntime().newRuntimeError("couldn't create document: " + ex);
+            throw asRuntimeError(runtime, "couldn't create document: ", ex);
         }
 
         Helpers.invoke(context, htmlDocument, "initialize", args);
@@ -109,46 +114,29 @@ public class HtmlDocument extends XmlDocument {
         return internalSubset;
     }
 
-    public static IRubyObject do_parse(ThreadContext context,
-                                       IRubyObject klass,
-                                       IRubyObject[] args) {
-        Ruby ruby = context.getRuntime();
-        Arity.checkArgumentCount(ruby, args, 4, 4);
-        HtmlDomParserContext ctx =
-            new HtmlDomParserContext(ruby, args[2], args[3]);
-        ctx.setInputSource(context, args[0], args[1]);
-        return ctx.parse(context, klass, args[1]);
-    }
-    
-    public void setDocumentNode(ThreadContext context, Node node) {
-        super.setNode(context, node);
-        Ruby runtime = context.getRuntime();
-        if (node != null) {
-            Document document = (Document)node;
-            document.normalize();
-            stabilzeAttrValue(document.getDocumentElement());
-        }
+    @Override
+    void init(Ruby runtime, Document document) {
+        stabilizeTextContent(document);
+        document.normalize();
         setInstanceVariable("@decorators", runtime.getNil());
+        if (document.getDocumentElement() != null) {
+            stabilizeAttrs(document.getDocumentElement());
+        }
     }
-    
-    private void stabilzeAttrValue(Node node) {
-        if (node == null) return;
+
+    private static void stabilizeAttrs(Node node) {
         if (node.hasAttributes()) {
             NamedNodeMap nodeMap = node.getAttributes();
             for (int i=0; i<nodeMap.getLength(); i++) {
                 Node n = nodeMap.item(i);
                 if (n instanceof Attr) {
-                    Attr attr = (Attr)n;
-                    String attrName = attr.getName();
-                    // not sure, but need to get value always before document is referred.
-                    // or lose attribute value
-                    String attrValue = attr.getValue(); // don't delete this line
+                    stabilizeAttr((Attr) n);
                 }
             }
         }
         NodeList children = node.getChildNodes();
         for (int i=0; i<children.getLength(); i++) {
-            stabilzeAttrValue(children.item(i));
+            stabilizeAttrs(children.item(i));
         }
     }
     
@@ -167,11 +155,11 @@ public class HtmlDocument extends XmlDocument {
      * Read the HTML document from +io+ with given +url+, +encoding+,
      * and +options+.  See Nokogiri::HTML.parse
      */
-    @JRubyMethod(meta = true, rest = true)
-    public static IRubyObject read_io(ThreadContext context,
-                                      IRubyObject cls,
-                                      IRubyObject[] args) {
-        return do_parse(context, cls, args);
+    @JRubyMethod(meta = true, required = 4)
+    public static IRubyObject read_io(ThreadContext context, IRubyObject klass, IRubyObject[] args) {
+        HtmlDomParserContext ctx = new HtmlDomParserContext(context.runtime, args[2], args[3]);
+        ctx.setIOInputSource(context, args[0], args[1]);
+        return ctx.parse(context, (RubyClass) klass, args[1]);
     }
 
     /*
@@ -181,10 +169,10 @@ public class HtmlDocument extends XmlDocument {
      * Read the HTML document contained in +string+ with given +url+, +encoding+,
      * and +options+.  See Nokogiri::HTML.parse
      */
-    @JRubyMethod(meta = true, rest = true)
-    public static IRubyObject read_memory(ThreadContext context,
-                                          IRubyObject cls,
-                                          IRubyObject[] args) {
-        return do_parse(context, cls, args);
+    @JRubyMethod(meta = true, required = 4)
+    public static IRubyObject read_memory(ThreadContext context, IRubyObject klass, IRubyObject[] args) {
+        HtmlDomParserContext ctx = new HtmlDomParserContext(context.runtime, args[2], args[3]);
+        ctx.setStringInputSource(context, args[0], args[1]);
+        return ctx.parse(context, (RubyClass) klass, args[1]);
     }
 }
