@@ -1,6 +1,6 @@
 require "helper"
 
-require 'uri'
+require "uri"
 
 module Nokogiri
   module XML
@@ -40,14 +40,14 @@ module Nokogiri
 
       # issue #838
       def test_document_with_invalid_prolog
-        doc = Nokogiri::XML '<? ?>'
+        doc = Nokogiri::XML "<? ?>"
         assert_empty doc.content
       end
 
       # issue #837
       def test_document_with_refentity
-        doc = Nokogiri::XML '&amp;'
-        assert_equal '', doc.content
+        doc = Nokogiri::XML "&amp;"
+        assert_equal "", doc.content
       end
 
       # issue 1060
@@ -55,18 +55,18 @@ module Nokogiri
         html = "<html><head></head><body><div>replace me</div></body></html>"
         doc = Nokogiri::XML::Document.parse(html)
         dup = doc.dup
-        assert_same dup, dup.at_css('div').document
+        assert_same dup, dup.at_css("div").document
 
         # should not raise an exception
-        dup.at_css('div').parse('<div>replaced</div>')
+        dup.at_css("div").parse("<div>replaced</div>")
       end
 
       # issue #835
       def test_manually_adding_reference_entities
         d = Nokogiri::XML::Document.new
-        root = Nokogiri::XML::Element.new('bar', d)
-        txt = Nokogiri::XML::Text.new('foo', d)
-        ent = Nokogiri::XML::EntityReference.new(d, '#8217')
+        root = Nokogiri::XML::Element.new("bar", d)
+        txt = Nokogiri::XML::Text.new("foo", d)
+        ent = Nokogiri::XML::EntityReference.new(d, "#8217")
         root << txt
         root << ent
         d << root
@@ -125,9 +125,9 @@ module Nokogiri
         </xml>
         eoxml
         if Nokogiri.jruby?
-          refute doc.xpath('//foo').first.namespace # assert that the namespace is nil
+          refute doc.xpath("//foo").first.namespace # assert that the namespace is nil
         end
-        refute_empty doc.xpath('//bar'), "bar wasn't found in the document" # bar should be part of the doc
+        refute_empty doc.xpath("//bar"), "bar wasn't found in the document" # bar should be part of the doc
       end
 
       def test_collect_namespaces
@@ -138,8 +138,8 @@ module Nokogiri
           </foo>
         </xml>
         eoxml
-        assert_equal({"xmlns"=>"hello", "xmlns:foo"=>"world"},
-          doc.collect_namespaces)
+        assert_equal({ "xmlns" => "hello", "xmlns:foo" => "world" },
+                     doc.collect_namespaces)
       end
 
       def test_subclass_initialize_modify # testing a segv
@@ -154,54 +154,54 @@ module Nokogiri
       end
 
       def test_create_text_node
-        txt = @xml.create_text_node 'foo'
+        txt = @xml.create_text_node "foo"
         assert_instance_of Nokogiri::XML::Text, txt
-        assert_equal 'foo', txt.text
+        assert_equal "foo", txt.text
         assert_equal @xml, txt.document
       end
 
       def test_create_text_node_with_block
-        @xml.create_text_node 'foo' do |txt|
+        @xml.create_text_node "foo" do |txt|
           assert_instance_of Nokogiri::XML::Text, txt
-          assert_equal 'foo', txt.text
+          assert_equal "foo", txt.text
           assert_equal @xml, txt.document
         end
       end
 
       def test_create_element
-        elm = @xml.create_element('foo')
+        elm = @xml.create_element("foo")
         assert_instance_of Nokogiri::XML::Element, elm
-        assert_equal 'foo', elm.name
+        assert_equal "foo", elm.name
         assert_equal @xml, elm.document
       end
 
       def test_create_element_with_block
-        @xml.create_element('foo') do |elm|
+        @xml.create_element("foo") do |elm|
           assert_instance_of Nokogiri::XML::Element, elm
-          assert_equal 'foo', elm.name
+          assert_equal "foo", elm.name
           assert_equal @xml, elm.document
         end
       end
 
       def test_create_element_with_attributes
-        elm = @xml.create_element('foo',:a => "1")
+        elm = @xml.create_element("foo", :a => "1")
         assert_instance_of Nokogiri::XML::Element, elm
         assert_instance_of Nokogiri::XML::Attr, elm.attributes["a"]
         assert_equal "1", elm["a"]
       end
 
       def test_create_element_with_namespace
-        elm = @xml.create_element('foo',:'xmlns:foo' => 'http://tenderlovemaking.com')
-        assert_equal 'http://tenderlovemaking.com', elm.namespaces['xmlns:foo']
+        elm = @xml.create_element("foo", :'xmlns:foo' => "http://tenderlovemaking.com")
+        assert_equal "http://tenderlovemaking.com", elm.namespaces["xmlns:foo"]
       end
 
       def test_create_element_with_hyphenated_namespace
-        elm = @xml.create_element('foo',:'xmlns:SOAP-ENC' => 'http://tenderlovemaking.com')
-        assert_equal 'http://tenderlovemaking.com', elm.namespaces['xmlns:SOAP-ENC']
+        elm = @xml.create_element("foo", :'xmlns:SOAP-ENC' => "http://tenderlovemaking.com")
+        assert_equal "http://tenderlovemaking.com", elm.namespaces["xmlns:SOAP-ENC"]
       end
 
       def test_create_element_with_content
-        elm = @xml.create_element('foo',"needs more xml/violence")
+        elm = @xml.create_element("foo", "needs more xml/violence")
         assert_equal "needs more xml/violence", elm.content
       end
 
@@ -240,19 +240,19 @@ module Nokogiri
       def test_create_internal_subset_on_existing_subset
         assert_not_nil @xml.internal_subset
         assert_raises(RuntimeError) do
-          @xml.create_internal_subset('staff', nil, 'staff.dtd')
+          @xml.create_internal_subset("staff", nil, "staff.dtd")
         end
       end
 
       def test_create_internal_subset
-        xml = Nokogiri::XML('<root />')
+        xml = Nokogiri::XML("<root />")
         assert_nil xml.internal_subset
 
-        xml.create_internal_subset('name', nil, 'staff.dtd')
+        xml.create_internal_subset("name", nil, "staff.dtd")
         ss = xml.internal_subset
-        assert_equal 'name', ss.name
+        assert_equal "name", ss.name
         assert_nil ss.external_id
-        assert_equal 'staff.dtd', ss.system_id
+        assert_equal "staff.dtd", ss.system_id
       end
 
       def test_external_subset
@@ -275,25 +275,25 @@ module Nokogiri
         assert @xml.external_subset
 
         assert_raises(RuntimeError) do
-          @xml.create_external_subset('staff', nil, 'staff.dtd')
+          @xml.create_external_subset("staff", nil, "staff.dtd")
         end
       end
 
       def test_create_external_subset
-        dtd = @xml.create_external_subset('staff', nil, 'staff.dtd')
+        dtd = @xml.create_external_subset("staff", nil, "staff.dtd")
         assert_nil dtd.external_id
-        assert_equal 'staff.dtd', dtd.system_id
-        assert_equal 'staff', dtd.name
+        assert_equal "staff.dtd", dtd.system_id
+        assert_equal "staff", dtd.name
         assert_equal dtd, @xml.external_subset
       end
 
       def test_version
-        assert_equal '1.0', @xml.version
+        assert_equal "1.0", @xml.version
       end
 
       def test_add_namespace
         assert_raise NoMethodError do
-          @xml.add_namespace('foo', 'bar')
+          @xml.add_namespace("foo", "bar")
         end
       end
 
@@ -322,7 +322,7 @@ module Nokogiri
       end
 
       def test_empty_node_converted_to_html_is_not_self_closing
-        doc = Nokogiri::XML('<a></a>')
+        doc = Nokogiri::XML("<a></a>")
         assert_equal "<a></a>", doc.inner_html
       end
 
@@ -333,15 +333,15 @@ module Nokogiri
 
       def test_add_child_fragment_with_single_node
         doc = Nokogiri::XML::Document.new
-        fragment = doc.fragment('<hello />')
+        fragment = doc.fragment("<hello />")
         doc.add_child fragment
-        assert_equal '/hello', doc.at('//hello').path
-        assert_equal 'hello', doc.root.name
+        assert_equal "/hello", doc.at("//hello").path
+        assert_equal "hello", doc.root.name
       end
 
       def test_add_child_fragment_with_multiple_nodes
         doc = Nokogiri::XML::Document.new
-        fragment = doc.fragment('<hello /><goodbye />')
+        fragment = doc.fragment("<hello /><goodbye />")
         assert_raises(RuntimeError) do
           doc.add_child fragment
         end
@@ -349,7 +349,7 @@ module Nokogiri
 
       def test_add_child_with_multiple_roots
         assert_raises(RuntimeError) do
-          @xml << Node.new('foo', @xml)
+          @xml << Node.new("foo", @xml)
         end
       end
 
@@ -361,16 +361,16 @@ module Nokogiri
       end
 
       def test_prepend
-        doc = Nokogiri::XML('<root>')
+        doc = Nokogiri::XML("<root>")
 
-        node_set = doc.root.prepend_child '<branch/>'
+        node_set = doc.root.prepend_child "<branch/>"
         assert_equal %w[branch], node_set.map(&:name)
 
-        branch = doc.at('//branch')
+        branch = doc.at("//branch")
 
         leaves = %w[leaf1 leaf2 leaf3]
         leaves.each { |name|
-          branch.prepend_child('<%s/>' % name)
+          branch.prepend_child("<%s/>" % name)
         }
         assert_equal leaves.length, branch.children.length
         assert_equal leaves.reverse, branch.children.map(&:name)
@@ -378,15 +378,15 @@ module Nokogiri
 
       def test_prepend_child_fragment_with_single_node
         doc = Nokogiri::XML::Document.new
-        fragment = doc.fragment('<hello />')
+        fragment = doc.fragment("<hello />")
         doc.prepend_child fragment
-        assert_equal '/hello', doc.at('//hello').path
-        assert_equal 'hello', doc.root.name
+        assert_equal "/hello", doc.at("//hello").path
+        assert_equal "hello", doc.root.name
       end
 
       def test_prepend_child_fragment_with_multiple_nodes
         doc = Nokogiri::XML::Document.new
-        fragment = doc.fragment('<hello /><goodbye />')
+        fragment = doc.fragment("<hello /><goodbye />")
         assert_raises(RuntimeError) do
           doc.prepend_child fragment
         end
@@ -394,7 +394,7 @@ module Nokogiri
 
       def test_prepend_child_with_multiple_roots
         assert_raises(RuntimeError) do
-          @xml.prepend_child Node.new('foo', @xml)
+          @xml.prepend_child Node.new("foo", @xml)
         end
       end
 
@@ -406,28 +406,28 @@ module Nokogiri
       end
 
       def test_move_root_to_document_with_no_root
-        sender = Nokogiri::XML('<root>foo</root>')
+        sender = Nokogiri::XML("<root>foo</root>")
         newdoc = Nokogiri::XML::Document.new
         newdoc.root = sender.root
       end
 
       def test_move_root_with_existing_root_gets_gcd
-        doc = Nokogiri::XML('<root>test</root>')
-        doc2 = Nokogiri::XML("<root>#{'x' * 5000000}</root>")
+        doc = Nokogiri::XML("<root>test</root>")
+        doc2 = Nokogiri::XML("<root>#{"x" * 5000000}</root>")
         doc2.root = doc.root
       end
 
       def test_validate
         if Nokogiri.uses_libxml?
-          assert_equal 44, @xml.validate.length
+          assert_equal 45, @xml.validate.length
         else
-          xml = Nokogiri::XML.parse(File.read(XML_FILE), XML_FILE) {|cfg| cfg.dtdvalid}
+          xml = Nokogiri::XML.parse(File.read(XML_FILE), XML_FILE) { |cfg| cfg.dtdvalid }
           assert_equal 40, xml.validate.length
         end
       end
 
       def test_validate_no_internal_subset
-        doc = Nokogiri::XML('<test/>')
+        doc = Nokogiri::XML("<test/>")
         assert_nil doc.validate
       end
 
@@ -439,11 +439,11 @@ module Nokogiri
         doc = Nokogiri::XML::Document.new
 
         assert_raises NoMethodError do
-          doc.default_namespace = 'http://innernet.com/'
+          doc.default_namespace = "http://innernet.com/"
         end
 
         assert_raises NoMethodError do
-          doc.add_namespace_definition('foo', 'bar')
+          doc.add_namespace_definition("foo", "bar")
         end
       end
 
@@ -523,11 +523,11 @@ module Nokogiri
       end
 
       def test_encoding=
-        @xml.encoding = 'UTF-8'
-        assert_match 'UTF-8', @xml.to_xml
+        @xml.encoding = "UTF-8"
+        assert_match "UTF-8", @xml.to_xml
 
-        @xml.encoding = 'EUC-JP'
-        assert_match 'EUC-JP', @xml.to_xml
+        @xml.encoding = "EUC-JP"
+        assert_match "EUC-JP", @xml.to_xml
       end
 
       def test_namespace_should_not_exist
@@ -542,12 +542,12 @@ module Nokogiri
         # is the correct thing to do.
         exception = RuntimeError
 
-        if !Nokogiri.uses_libxml? || (Nokogiri.uses_libxml? && Nokogiri::VERSION_INFO['libxml']['platform'] == 'jruby')
+        if !Nokogiri.uses_libxml? || (Nokogiri.uses_libxml? && Nokogiri::VERSION_INFO["libxml"]["platform"] == "jruby")
           exception = Nokogiri::XML::XPath::SyntaxError
         end
 
         assert_raises(exception) {
-          @xml.xpath('//name[foo()]')
+          @xml.xpath("//name[foo()]")
         }
       end
 
@@ -575,15 +575,15 @@ module Nokogiri
             </foo>
           </root>
         eoxml
-        assert_equal 1, doc.css('xmlns|foo').length
-        assert_equal 1, doc.css('foo').length
-        assert_equal 0, doc.css('|foo').length
-        assert_equal 1, doc.xpath('//xmlns:foo').length
-        assert_equal 1, doc.search('xmlns|foo').length
-        assert_equal 1, doc.search('//xmlns:foo').length
-        assert doc.at('xmlns|foo')
-        assert doc.at('//xmlns:foo')
-        assert doc.at('foo')
+        assert_equal 1, doc.css("xmlns|foo").length
+        assert_equal 1, doc.css("foo").length
+        assert_equal 0, doc.css("|foo").length
+        assert_equal 1, doc.xpath("//xmlns:foo").length
+        assert_equal 1, doc.search("xmlns|foo").length
+        assert_equal 1, doc.search("//xmlns:foo").length
+        assert doc.at("xmlns|foo")
+        assert doc.at("//xmlns:foo")
+        assert doc.at("foo")
       end
 
       def test_xmlns_is_registered_for_nodesets
@@ -596,15 +596,15 @@ module Nokogiri
             </foo>
           </root>
         eoxml
-        assert_equal 1, doc.css('xmlns|foo').css('xmlns|bar').length
-        assert_equal 1, doc.css('foo').css('bar').length
-        assert_equal 1, doc.xpath('//xmlns:foo').xpath('./xmlns:bar').length
-        assert_equal 1, doc.search('xmlns|foo').search('xmlns|bar').length
-        assert_equal 1, doc.search('//xmlns:foo').search('./xmlns:bar').length
+        assert_equal 1, doc.css("xmlns|foo").css("xmlns|bar").length
+        assert_equal 1, doc.css("foo").css("bar").length
+        assert_equal 1, doc.xpath("//xmlns:foo").xpath("./xmlns:bar").length
+        assert_equal 1, doc.search("xmlns|foo").search("xmlns|bar").length
+        assert_equal 1, doc.search("//xmlns:foo").search("./xmlns:bar").length
       end
 
       def test_to_xml_with_indent
-        doc = Nokogiri::XML('<root><foo><bar/></foo></root>')
+        doc = Nokogiri::XML("<root><foo><bar/></foo></root>")
         doc = Nokogiri::XML(doc.to_xml(:indent => 5))
 
         assert_indent 5, doc
@@ -612,18 +612,17 @@ module Nokogiri
 
       def test_write_xml_to_with_indent
         io = StringIO.new
-        doc = Nokogiri::XML('<root><foo><bar/></foo></root>')
+        doc = Nokogiri::XML("<root><foo><bar/></foo></root>")
         doc.write_xml_to io, :indent => 5
         io.rewind
         doc = Nokogiri::XML(io.read)
         assert_indent 5, doc
       end
 
-      # wtf...  osx's libxml sucks.
-      unless !Nokogiri.uses_libxml? || Nokogiri::LIBXML_VERSION =~ /^2\.6\./
+      if ! Nokogiri.uses_libxml?("~> 2.6.0")
         def test_encoding
-          xml = Nokogiri::XML(File.read(XML_FILE), XML_FILE, 'UTF-8')
-          assert_equal 'UTF-8', xml.encoding
+          xml = Nokogiri::XML(File.read(XML_FILE), XML_FILE, "UTF-8")
+          assert_equal "UTF-8", xml.encoding
         end
       end
 
@@ -652,17 +651,17 @@ module Nokogiri
 
       def test_strict_document_throws_syntax_error
         assert_raises(Nokogiri::XML::SyntaxError) {
-          Nokogiri::XML('<foo><bar></foo>', nil, nil, 0)
+          Nokogiri::XML("<foo><bar></foo>", nil, nil, 0)
         }
 
         assert_raises(Nokogiri::XML::SyntaxError) {
-          Nokogiri::XML('<foo><bar></foo>') { |cfg|
+          Nokogiri::XML("<foo><bar></foo>") { |cfg|
             cfg.strict
           }
         }
 
         assert_raises(Nokogiri::XML::SyntaxError) {
-          Nokogiri::XML(StringIO.new('<foo><bar></foo>')) { |cfg|
+          Nokogiri::XML(StringIO.new("<foo><bar></foo>")) { |cfg|
             cfg.strict
           }
         }
@@ -675,7 +674,7 @@ module Nokogiri
 
       def test_url
         assert @xml.url
-        assert_equal XML_FILE, URI.unescape(@xml.url).sub('file:///', '')
+        assert_equal XML_FILE, @xml.url
       end
 
       def test_document_parent
@@ -687,44 +686,72 @@ module Nokogiri
 
       def test_document_name
         xml = Nokogiri::XML(File.read(XML_FILE), XML_FILE)
-        assert_equal 'document', xml.name
+        assert_equal "document", xml.name
       end
 
       def test_parse_can_take_io
         xml = nil
-        File.open(XML_FILE, 'rb') { |f|
+        File.open(XML_FILE, "rb") { |f|
           xml = Nokogiri::XML(f)
         }
         assert xml.xml?
-        set = xml.search('//employee')
+        assert_equal XML_FILE, xml.url
+        set = xml.search("//employee")
         assert set.length > 0
       end
 
       def test_parsing_empty_io
-        doc = Nokogiri::XML.parse(StringIO.new(''))
+        doc = Nokogiri::XML.parse(StringIO.new(""))
         refute_nil doc
       end
 
       def test_parse_works_with_an_object_that_responds_to_read
         klass = Class.new do
-          def read *args
-            "<div>foo</div>"
+          def initialize
+            @contents = StringIO.new("<div>foo</div>")
+          end
+
+          def read(*args)
+            @contents.read(*args)
           end
         end
 
         doc = Nokogiri::XML.parse klass.new
-        doc.at_css("div").content.must_equal("foo")
+        assert_equal "foo", doc.at_css("div").content
+      end
+
+      def test_parse_works_with_an_object_that_responds_to_path
+        xml = String.new("<root><sub>hello</sub></root>")
+        def xml.path
+          "/i/should/be/the/document/url"
+        end
+
+        doc = Nokogiri::XML.parse(xml)
+
+        assert_equal "/i/should/be/the/document/url", doc.url
+      end
+
+      # issue #1821, #2110
+      def test_parse_can_take_pathnames
+        assert(File.size(XML_ATOM_FILE) > 4096) # file must be big enough to trip the read callback more than once
+
+        doc = Nokogiri::XML.parse(Pathname.new(XML_ATOM_FILE))
+
+        # an arbitrary assertion on the structure of the document
+        assert_equal 20, doc.xpath("/xmlns:feed/xmlns:entry/xmlns:author",
+                                   "xmlns" => "http://www.w3.org/2005/Atom").length
+        assert_equal XML_ATOM_FILE, doc.url
       end
 
       def test_search_on_empty_documents
         doc = Nokogiri::XML::Document.new
-        ns = doc.search('//foo')
+        ns = doc.search("//foo")
         assert_equal 0, ns.length
 
-        ns = doc.css('foo')
+        ns = doc.css("foo")
         assert_equal 0, ns.length
 
-        ns = doc.xpath('//foo')
+        ns = doc.xpath("//foo")
         assert_equal 0, ns.length
       end
 
@@ -743,9 +770,9 @@ module Nokogiri
         document = Nokogiri::XML(xml)
         assert_kind_of Nokogiri::XML::Document, document
 
-        assert_equal 3, document.xpath('.//div', './/p').length
-        assert_equal 3, document.css('.title', '.content', 'p').length
-        assert_equal 3, document.search('.//div', 'p.blah').length
+        assert_equal 3, document.xpath(".//div", ".//p").length
+        assert_equal 3, document.css(".title", ".content", "p").length
+        assert_equal 3, document.search(".//div", "p.blah").length
       end
 
       def test_bad_xpath_raises_syntax_error
@@ -762,22 +789,22 @@ module Nokogiri
         eoxml
 
         ctx = Nokogiri::XML::XPathContext.new(doc)
-        ctx.register_ns 'tenderlove', 'http://tenderlovemaking.com/'
-        set = ctx.evaluate('//tenderlove:foo')
+        ctx.register_ns "tenderlove", "http://tenderlovemaking.com/"
+        set = ctx.evaluate("//tenderlove:foo")
         assert_equal 1, set.length
-        assert_equal 'foo', set.first.name
+        assert_equal "foo", set.first.name
 
         # It looks like only the URI is important:
         ctx = Nokogiri::XML::XPathContext.new(doc)
-        ctx.register_ns 'america', 'http://tenderlovemaking.com/'
-        set = ctx.evaluate('//america:foo')
+        ctx.register_ns "america", "http://tenderlovemaking.com/"
+        set = ctx.evaluate("//america:foo")
         assert_equal 1, set.length
-        assert_equal 'foo', set.first.name
+        assert_equal "foo", set.first.name
 
         # Its so important that a missing slash will cause it to return nothing
         ctx = Nokogiri::XML::XPathContext.new(doc)
-        ctx.register_ns 'america', 'http://tenderlovemaking.com'
-        set = ctx.evaluate('//america:foo')
+        ctx.register_ns "america", "http://tenderlovemaking.com"
+        set = ctx.evaluate("//america:foo")
         assert_equal 0, set.length
       end
 
@@ -790,25 +817,25 @@ module Nokogiri
       end
 
       def test_singleton_methods
-        assert node_set = @xml.search('//name')
+        assert node_set = @xml.search("//name")
         assert node_set.length > 0
         node = node_set.first
         def node.test
-          'test'
+          "test"
         end
-        assert node_set = @xml.search('//name')
-        assert_equal 'test', node_set.first.test
+        assert node_set = @xml.search("//name")
+        assert_equal "test", node_set.first.test
       end
 
       def test_multiple_search
-        assert node_set = @xml.search('//employee', '//name')
-        employees = @xml.search('//employee')
-        names = @xml.search('//name')
+        assert node_set = @xml.search("//employee", "//name")
+        employees = @xml.search("//employee")
+        names = @xml.search("//name")
         assert_equal(employees.length + names.length, node_set.length)
       end
 
       def test_node_set_index
-        assert node_set = @xml.search('//employee')
+        assert node_set = @xml.search("//employee")
 
         assert_equal(5, node_set.length)
         assert node_set[4]
@@ -816,12 +843,12 @@ module Nokogiri
       end
 
       def test_search
-        assert node_set = @xml.search('//employee')
+        assert node_set = @xml.search("//employee")
 
         assert_equal(5, node_set.length)
 
         node_set.each do |node|
-          assert_equal('employee', node.name)
+          assert_equal("employee", node.name)
         end
       end
 
@@ -833,7 +860,7 @@ module Nokogiri
       def test_dup
         dup = @xml.dup
         assert_instance_of Nokogiri::XML::Document, dup
-        assert dup.xml?, 'duplicate should be xml'
+        assert dup.xml?, "duplicate should be xml"
       end
 
       def test_new
@@ -851,9 +878,9 @@ module Nokogiri
         assert doc.xml?
         assert_nil doc.root
         node = Nokogiri::XML::Node.new("b", doc) { |n|
-          n.content = 'hello world'
+          n.content = "hello world"
         }
-        assert_equal('hello world', node.content)
+        assert_equal("hello world", node.content)
         doc.root = node
         assert_equal(node, doc.root)
       end
@@ -880,9 +907,9 @@ module Nokogiri
         assert_equal 1, doc.xpath("//x:foo", "x" => "http://c.flavorjon.es/").length
         assert_match %r{foo c:attr}, doc.to_xml
         doc.at_xpath("//x:foo", "x" => "http://c.flavorjon.es/").tap do |node|
-          assert_nil          node["attr"]
+          assert_nil node["attr"]
           assert_equal "attr-value", node["c:attr"]
-          assert_nil          node.attribute_with_ns("attr", nil)
+          assert_nil node.attribute_with_ns("attr", nil)
           assert_equal "attr-value", node.attribute_with_ns("attr", "http://c.flavorjon.es/").value
           assert_equal "attr-value", node.attributes["attr"].value
         end
@@ -898,9 +925,9 @@ module Nokogiri
         assert_match %r{foo attr}, doc.to_xml
         doc.at_xpath("//container/foo").tap do |node|
           assert_equal "attr-value", node["attr"]
-          assert_nil          node["c:attr"]
+          assert_nil node["c:attr"]
           assert_equal "attr-value", node.attribute_with_ns("attr", nil).value
-          assert_nil          node.attribute_with_ns("attr", "http://c.flavorjon.es/")
+          assert_nil node.attribute_with_ns("attr", "http://c.flavorjon.es/")
           assert_equal "attr-value", node.attributes["attr"].value # doesn't change!
         end
       end
@@ -914,7 +941,7 @@ module Nokogiri
 
         util_decorate(@xml, decorator)
 
-        assert @xml.search('//@street').first.respond_to?(:test_method)
+        assert @xml.search("//@street").first.respond_to?(:test_method)
       end
 
       def test_subset_is_decorated
@@ -925,11 +952,11 @@ module Nokogiri
         util_decorate(@xml, x)
 
         assert @xml.respond_to?(:awesome!)
-        assert node_set = @xml.search('//staff')
+        assert node_set = @xml.search("//staff")
         assert node_set.respond_to?(:awesome!)
-        assert subset = node_set.search('.//employee')
+        assert subset = node_set.search(".//employee")
         assert subset.respond_to?(:awesome!)
-        assert sub_subset = node_set.search('.//name')
+        assert sub_subset = node_set.search(".//name")
         assert sub_subset.respond_to?(:awesome!)
       end
 
@@ -941,7 +968,7 @@ module Nokogiri
         util_decorate(@xml, x)
 
         assert @xml.respond_to?(:awesome!)
-        assert node_set = @xml.search('//employee')
+        assert node_set = @xml.search("//employee")
         assert node_set.respond_to?(:awesome!)
         node_set.each do |node|
           assert node.respond_to?(:awesome!), node.class
@@ -952,7 +979,7 @@ module Nokogiri
 
       if Nokogiri.jruby?
         def wrap_java_document
-          require 'java'
+          require "java"
           factory = javax.xml.parsers.DocumentBuilderFactory.newInstance
           builder = factory.newDocumentBuilder
           document = builder.newDocument
@@ -963,9 +990,9 @@ module Nokogiri
       end
 
       def test_java_integration
-        skip("Ruby doesn't have the wrap method") unless Nokogiri.jruby?
+        skip("CRuby doesn't have the Document#wrap method") unless Nokogiri.jruby?
         noko_doc = wrap_java_document
-        assert_equal 'foo', noko_doc.root.name
+        assert_equal "foo", noko_doc.root.name
 
         noko_doc = Nokogiri::XML(<<eoxml)
 <foo xmlns='hello'>
@@ -974,11 +1001,11 @@ module Nokogiri
 eoxml
         dom = noko_doc.to_java
         assert dom.kind_of? org.w3c.dom.Document
-        assert_equal 'foo', dom.getDocumentElement().getTagName()
+        assert_equal "foo", dom.getDocumentElement().getTagName()
       end
 
       def test_add_child
-        skip("Ruby doesn't have the wrap method") unless Nokogiri.jruby?
+        skip("CRuby doesn't have the Document#wrap method") unless Nokogiri.jruby?
         doc = wrap_java_document
         doc.root.add_child "<bar />"
       end
@@ -988,6 +1015,61 @@ eoxml
         Nokogiri::XML f
         f.close
       end
+
+      describe "XML::Document.parse" do
+        # establish baseline behavior for HTML document behavior in
+        # https://github.com/sparklemotion/nokogiri/issues/2130
+        # (see similar tests in test/html/test_document.rb)
+        let(:xml_strict) do
+          Nokogiri::XML::ParseOptions.new(Nokogiri::XML::ParseOptions::DEFAULT_XML).norecover
+        end
+
+        it "sets the test up correctly" do
+          assert(xml_strict.strict?)
+        end
+
+        describe "read memory" do
+          let(:input) { "<html><body><div" }
+
+          describe "strict parsing" do
+            let(:parse_options) { xml_strict }
+
+            it "raises exception on parse error" do
+              assert_raises Nokogiri::SyntaxError do
+                Nokogiri::XML.parse(input, nil, nil, parse_options)
+              end
+            end
+          end
+
+          describe "default options" do
+            it "does not raise exception on parse error" do
+              doc = Nokogiri::XML.parse(input)
+              assert_operator(doc.errors.length, :>, 0)
+            end
+          end
+        end
+
+        describe "read io" do
+          let(:input) { StringIO.new("<html><body><div") }
+
+          describe "strict parsing" do
+            let(:parse_options) { xml_strict }
+
+            it "raises exception on parse error" do
+              assert_raises Nokogiri::SyntaxError do
+                Nokogiri::XML.parse(input, nil, nil, parse_options)
+              end
+            end
+          end
+
+          describe "default options" do
+            it "does not raise exception on parse error" do
+              doc = Nokogiri::XML.parse(input)
+              assert_operator(doc.errors.length, :>, 0)
+            end
+          end
+        end
+      end
     end
   end
 end
