@@ -116,12 +116,11 @@ module Nokogiri
       def test_node_context_parsing_of_malformed_html_fragment_without_recover_is_not_corrected
         doc = HTML.parse "<html><body><div></div></body></html>"
         context_node = doc.at_css "div"
-        nodeset = context_node.parse("<div </div>") do |options|
-          options.strict
+        assert_raises(Nokogiri::XML::SyntaxError) do
+          context_node.parse("<div </div>") do |options|
+            options.strict
+          end
         end
-
-        assert_equal 1, doc.errors.length
-        assert_equal 0, nodeset.length
       end
 
       def test_parse_error_list
@@ -368,9 +367,19 @@ module Nokogiri
       end
 
       def test_document_compare
-        skip "underlying libxml2 behavior changed in libxml2@a005199"
         nodes = @xml.xpath('//employee')
-        assert_equal(-1, (nodes.first <=> @xml)) # post-libxml2@a005199, returns 1
+        result = (nodes.first <=> @xml)
+
+        # Note that this behavior was changed in libxml@a005199 starting in v2.9.5.
+        #
+        # But that fix was backported by debian (and other distros?) into their v2.9.4 fork so we
+        # can't use version to detect what's going on here. Instead just shrug if we're using system
+        # libraries and the result is -1.
+        if (Nokogiri::VersionInfo.instance.libxml2_using_system?)
+          assert(result == 1 || result == -1)
+        else
+          assert_equal(1, result)
+        end
       end
 
       def test_different_document_compare
@@ -579,15 +588,6 @@ module Nokogiri
         assert_equal @xml.to_xml, io.read
       end
 
-      def test_attribute_with_symbol
-        assert_equal 'Yes', @xml.css('address').first[:domestic]
-      end
-
-      def test_non_existent_attribute_should_return_nil
-        node = @xml.root.first_element_child
-        assert_nil node.attribute('type')
-      end
-
       def test_write_to_with_block
         called = false
         io = StringIO.new
@@ -640,22 +640,6 @@ module Nokogiri
         assert_equal 'b', node_b.name
       end
 
-      def test_values
-        assert_equal %w{ Yes Yes }, @xml.xpath('//address')[1].values
-      end
-
-      def test_keys
-        assert_equal %w{ domestic street }, @xml.xpath('//address')[1].keys
-      end
-
-      def test_each
-        attributes = []
-        @xml.xpath('//address')[1].each do |key, value|
-          attributes << [key, value]
-        end
-        assert_equal [['domestic', 'Yes'], ['street', 'Yes']], attributes
-      end
-
       def test_new
         assert node = Nokogiri::XML::Node.new('input', @xml)
         assert_equal 1, node.node_type
@@ -682,132 +666,6 @@ module Nokogiri
         assert entity_decl.read_only?
       end
 
-      def test_remove_attribute
-        address = @xml.xpath('/staff/employee/address').first
-        assert_equal 'Yes', address['domestic']
-        attr = address.attributes['domestic']
-
-        returned_attr = address.remove_attribute 'domestic'
-        assert_nil address['domestic']
-        assert_equal attr, returned_attr
-      end
-
-      def test_remove_attribute_when_not_found
-        address = @xml.xpath('/staff/employee/address').first
-        attr = address.remove_attribute 'not-an-attribute'
-        assert_nil attr
-      end
-
-      def test_attribute_setter_accepts_non_string
-        address = @xml.xpath("/staff/employee/address").first
-        assert_equal "Yes", address[:domestic]
-        address[:domestic] = "Altered Yes"
-        assert_equal "Altered Yes", address[:domestic]
-      end
-
-      def test_attribute_accessor_accepts_non_string
-        address = @xml.xpath("/staff/employee/address").first
-        assert_equal "Yes", address["domestic"]
-        assert_equal "Yes", address[:domestic]
-      end
-
-      def test_empty_attribute_reading
-        node = Nokogiri::XML '<foo empty="" whitespace="  "/>'
-
-        assert_equal '', node.root['empty']
-        assert_equal '  ', node.root['whitespace']
-      end
-
-      def test_delete
-        address = @xml.xpath('/staff/employee/address').first
-        assert_equal 'Yes', address['domestic']
-        address.delete 'domestic'
-        assert_nil address['domestic']
-      end
-
-      def test_classes
-        xml = Nokogiri::XML(<<-eoxml)
-        <div>
-          <p class=" foo  bar foo ">test</p>
-          <p class="">test</p>
-        </div>
-        eoxml
-        div = xml.at_xpath('//div')
-        p1, p2 = xml.xpath('//p')
-
-        assert_equal [], div.classes
-        assert_equal %w[foo bar foo], p1.classes
-        assert_equal [], p2.classes
-      end
-
-      def test_add_class
-        xml = Nokogiri::XML(<<-eoxml)
-        <div>
-          <p class=" foo  bar foo ">test</p>
-          <p class="">test</p>
-        </div>
-        eoxml
-        div = xml.at_xpath('//div')
-        p1, p2 = xml.xpath('//p')
-
-        assert_same div, div.add_class('main')
-        assert_equal 'main', div['class']
-
-        assert_same p1, p1.add_class('baz foo')
-        assert_equal 'foo bar foo baz', p1['class']
-
-        assert_same p2, p2.add_class('foo baz foo')
-        assert_equal 'foo baz foo', p2['class']
-      end
-
-      def test_append_class
-        xml = Nokogiri::XML(<<-eoxml)
-        <div>
-          <p class=" foo  bar foo ">test</p>
-          <p class="">test</p>
-        </div>
-        eoxml
-        div = xml.at_xpath('//div')
-        p1, p2 = xml.xpath('//p')
-
-        assert_same div, div.append_class('main')
-        assert_equal 'main', div['class']
-
-        assert_same p1, p1.append_class('baz foo')
-        assert_equal 'foo bar foo baz foo', p1['class']
-
-        assert_same p2, p2.append_class('foo baz foo')
-        assert_equal 'foo baz foo', p2['class']
-      end
-
-      def test_remove_class
-        xml = Nokogiri::XML(<<-eoxml)
-        <div>
-          <p class=" foo  bar baz foo ">test</p>
-          <p class=" foo  bar baz foo ">test</p>
-          <p class="foo foo">test</p>
-          <p class="">test</p>
-        </div>
-        eoxml
-        div = xml.at_xpath('//div')
-        p1, p2, p3, p4 = xml.xpath('//p')
-
-        assert_same div, div.remove_class('main')
-        assert_nil div['class']
-
-        assert_same p1, p1.remove_class('bar baz')
-        assert_equal 'foo foo', p1['class']
-
-        assert_same p2, p2.remove_class()
-        assert_nil p2['class']
-
-        assert_same p3, p3.remove_class('foo')
-        assert_nil p3['class']
-
-        assert_same p4, p4.remove_class('foo')
-        assert_nil p4['class']
-      end
-
       def test_set_content_with_symbol
         node = @xml.at('//name')
         node.content = :foo
@@ -914,19 +772,6 @@ module Nokogiri
         assert_equal('false', node['foo'])
       end
 
-      def test_attributes
-        assert node = @xml.search('//address').first
-        assert_nil(node['asdfasdfasdf'])
-        assert_equal('Yes', node['domestic'])
-
-        assert node = @xml.search('//address')[2]
-        attr = node.attributes
-        assert_equal 2, attr.size
-        assert_equal 'Yes', attr['domestic'].value
-        assert_equal 'Yes', attr['domestic'].to_s
-        assert_equal 'No', attr['street'].value
-      end
-
       def test_path
         assert set = @xml.search('//employee')
         assert node = set.first
@@ -1222,6 +1067,14 @@ EOXML
         assert_equal 2, node.line
       end
 
+      def test_set_line
+        skip "Only supported with libxml2" unless Nokogiri.uses_libxml?
+        document = Nokogiri::XML::Document.new
+        node = document.create_element('a')
+        node.line = 54321
+        assert_equal 54321, node.line
+      end
+
       def test_xpath_results_have_document_and_are_decorated
         x = Module.new do
           def awesome! ; end
@@ -1356,11 +1209,12 @@ eoxml
       end
 
       def test_text_node_robustness_gh1426
+        skip "No need to test libxml-ruby workarounds on JRuby" if Nokogiri.jruby?
         # notably, the original bug report was about libxml-ruby interactions
         # this test should blow up under valgrind if we regress on libxml-ruby workarounds
-        message = "<h2>BOOM!</h2>"
+        message = "<section><h2>BOOM!</h2></section>"
         10_000.times do
-          node = Nokogiri::HTML::DocumentFragment.parse(message)
+          node = Nokogiri::HTML::DocumentFragment.parse(message).at_css("h2")
           node.add_previous_sibling(Nokogiri::XML::Text.new('before', node.document))
           node.add_next_sibling(Nokogiri::XML::Text.new('after', node.document))
         end
